##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Test that some Zope interfaces are supported properly."""

import unittest
import ZODB # for Persistent
from Products.ParsedXML import ParsedXML
from Products.ParsedXML.StrIO import StringIO


def assertSize(doc):
    "assert that the document size what's reported by len"
    gs = doc.get_size()
    l = len(str(doc))
    assert gs == l, "get_size reports %d while len reports %d" % (gs, l)
    
class GetSizeTestCase(unittest.TestCase):
    "test that get_size works.  We only test on the persistent Document."

    def setUp(self):
        self.document = ParsedXML.ParsedXML('foo')

    def checkGetSize(self):
        "assert that get_size works"
        assertSize(self.document)

    def checkGetSizeParse(self):
        "assert that get_size works after a parse"        
        inStr = '<spam><eggs attr1="foo"><ham/>text</eggs></spam>'
        self.document.parseXML(StringIO(inStr))
        assertSize(self.document)        
        self.document.documentElement.parseXML(StringIO(inStr))
        assertSize(self.document)        

    def checkGetSizeDOMMethods(self):
        "assert that get_size works after some DOM method manipulations"
        self.document.documentElement.appendChild(
            self.document.createElement('spam'))
        assertSize(self.document)
        self.document.documentElement.appendChild(
            self.document.createTextNode('spam'))
        assertSize(self.document)
        self.document.documentElement.appendChild(
            self.document.createTextNode('spam'))
        assertSize(self.document)
        self.document.normalize()
        assertSize(self.document)
        self.document.documentElement.setAttribute('eggs', 'ham')
        assertSize(self.document)

    def checkgetSizeDOMAttributess(self):
        "assert that get_size works after some DOM attribute manipulations"
        self.document.documentElement.appendChild(
            self.document.createTextNode('spam'))
        self.document.documentElement.firstChild.data = "spamspamspam"
        assertSize(self.document)
        self.document.documentElement.setAttribute('eggs', 'ham')
        self.document.documentElement.attributes.item(0).value = 'spam'
        assertSize(self.document)
        
def test_suite():
    """Return a test suite for the Zope testing framework."""

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GetSizeTestCase, 'check'))
    return suite

def main():
    unittest.TextTestRunner().run(test_suite())

if __name__ == "__main__":
    main()
